package Tools;

@ISA = qw(Exporter);

use FindBin qw($RealBin);
use lib sprintf("%s", $RealBin);
use lib sprintf("%s/../lib", $RealBin);

use Data::Dumper;
$Data::Dumper::Indent = 1;
$Data::Dumper::Maxdepth = 2;

use IO::File;

@EXPORT = qw(&stackTrace &error &rep2str &dumper &panic &getFromSocket &fields &load_file &save_file &tableExists &buildsearch);

#returns a string containg the stacktrace
# ------------------
sub stackTrace {
# ------------------
    my $result;
    my $l;
    my $s;
    my $n = 0;
    $result .= "=========== top of stack =========\n";
    do {
	my ($package, $filename, $line, $subroutine) = caller($n++);
	$l = $s;
	$s = sprintf("%s:%s (%s)", $filename||"", $line||"", $subroutine||"");
	if ($s ne ":") {
	    $result .= "$s\n";
	}
    } while (!defined $l || $s ne $l);
    $result .= "=========== end of stack =========\n";
}

# ------------------
sub error {
# ------------------
    my $self = shift if(scalar @_ > 1);
    my $msg = shift;

    # Register errCallback error(sub{...})
    if(ref $msg eq 'CODE') {
        $self->{errCallback} = $msg;
        return 1;
    }

    unless($msg) {
        my $err = $__PACKAGE__::error;
        $__PACKAGE__::error = '';
        return $err;
    }
    $__PACKAGE__::{error} = sprintf($msg);

    if(defined $self and defined $self->{errCallback} and ref $self->{errCallback} eq 'CODE') {
        $msg .= "\n\n".&stackTrace;
        &{$self->{errCallback}}($msg);
    } else {
        warn $msg;
        warn &stackTrace;
      }

    return undef;
}

# ------------------
sub panic {
# ------------------
    my $self = shift if(scalar @_ > 1);
    my $msg = shift;

    # Register errCallback error(sub{...})
    if(ref $msg eq 'CODE') {
        $self->{errCallback} = $msg;
        return 1;
    }

    unless($msg) {
        my $err = $__PACKAGE__::error;
        $__PACKAGE__::error = '';
        return $err;
    }
    $__PACKAGE__::{error} = sprintf($msg);

    if(defined $self and defined $self->{errCallback} and ref $self->{errCallback} eq 'CODE') {
#        $msg .= "\n\n".&stackTrace;
        &{$self->{errCallback}}($msg);
        exit;
    } else {
        printf("%s\n", $msg);
#        warn &stackTrace;
        exit;
    }
}

# ------------------
sub getFromSocket {
# ------------------
    my $sock = shift or return undef;

    my (@Data, $Line);
	do {
		$Line = <$sock> || 0;
		$Line =~ s/\r\n//g;
		push(@Data, $Line);
	} while($Line);

    return \@Data;
}


# ------------------
sub rep2str {
# ------------------
    my %args = @_;
    my $text = $args{-text};
    $text =~ s/\<(.+?)\>/$args{"-$1"}/sig;
    return $text;
}

# ------------------
sub dumper {
# ------------------
    my $var = shift || '<undef>';
    my $args = @_;

    $Data::Dumper::Maxdepth = $args->{d}
        if($args->{d});
    $Data::Dumper::Indent = $args->{i}
        if($args->{d});

    warn Dumper( $var );
}

# ------------------
sub fields {
# ------------------
    my $dbh = shift || return error ('No DBH!' );
    my $str = shift || return error ('No SQL!');

    $str =~ s/order\s+by.+//sig;
    my $sql = sprintf('%s %s 0 = 1', $str, ($str =~ /where/i ? 'AND' : 'WHERE'));
	my $sth = $dbh->prepare($sql) or return error("$DBI::errstr - $sql");
	$sth->execute or return error("$DBI::errstr - $sql");
	my $fields = $sth->{'NAME'};
    return $fields;
}

# ------------------
sub tableExists {
# ------------------
    my $dbh = shift || return error ('No DBH!' );
    my $name = shift || return error ('No Table!');

    my $erg = $dbh->selectall_arrayref('show tables');
    for(@$erg) {
        return 1 if($name eq $_->[0]);
    }
    return 0;
}

#--------------------------------------------------------
sub load_file {
#--------------------------------------------------------
	my $file = shift || die "Kein File bei Loader $!";
	my $fh = IO::File->new("< $file")
	    or return error("Can't open File $file $! ");
	my $data;
	while ( defined (my $l = <$fh>) ) {
	        $data .= $l;
	}
	$fh->close;
	return $data;
}

#--------------------------------------------------------
sub save_file {
#--------------------------------------------------------
    my ($file, $data) = @_;
	return unless($file);
	my $fhi = new IO::File("> $file")
	    or return error("Can't write!");
	print $fhi $data;
	$fhi->close;

    return $file
}


#--------------------------------------------------------
sub buildsearch {
#--------------------------------------------------------
    my ($InFields, $Search) = @_;
    my $ss = {};
    foreach my $su (split(/\s*,\s*/, $Search)) {
        $su =~ s/\'/\./sg;

        # Split inFields and make a sql where clausel
        # if field with minus at top, then not rlike
        foreach my $f (split(/\s*,\s*/, $InFields)) {
            my $searchtext = $su;
            if($searchtext =~ s/^\-//) {
                push(@{$ss->{$su}}, "$f NOT RLIKE '$searchtext'");
            } else {
                push(@{$ss->{$su}}, "$f RLIKE '$searchtext'");
            }
        }
    }

    my @fieldsearch;
    foreach my $su (keys %$ss) {
        my $type = ($su =~ /^\-/) ? 'AND' : 'OR';
        push(@fieldsearch, "(" . join(" $type ", @{$ss->{$su}}) . ")");
    }
	return join('AND ', @fieldsearch);
}

1;
