package XXV::MODULES::AUTOTIMER;

use strict;

use Tools;
use Event::File;
use Locale::gettext;


# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'AUTOTIMER',
        Description => gettext('This module search for epg Entrys with an user defined text and create new timer.'),
        Version => '0.01',
        Date => '09.09.2004',
        Author => 'xpix',
        Preferences => {
            interval => {
                description => gettext('The interval in seconds to search for timers'),
                default     => 300,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            astatus => {
                description => gettext('Status from Autotimers'),
                short       => 'as',
                callback    => sub{ $obj->status(@_) },
            },
            anew => {
                description => gettext("Create a autotimer"),
                short       => 'an',
                callback    => sub{ $obj->autotimerCreate(@_) },
                Level       => 'user',
            },
            adelete => {
                description => gettext("Delete a autotimer 'aid'"),
                short       => 'ad',
                callback    => sub{ $obj->autotimerDelete(@_) },
                Level       => 'user',
            },
            aedit => {
                description => gettext("Edit a autotimer 'aid'"),
                short       => 'ae',
                callback    => sub{ $obj->autotimerEdit(@_) },
                Level       => 'user',
            },
            alist => {
                description => gettext("List the autotimer 'aid'"),
                short       => 'al',
                callback    => sub{ $obj->list(@_) },
            },
            aupdate => {
                description => gettext("Start the autotimer process"),
                short       => 'au',
                callback    => sub{ $obj->autotimer(@_) },
            },
            aimport => {
                description => gettext("Import autotimer from vdradmind.at"),
                short       => 'ai',
                callback    => sub{ $obj->autotimerimport(@_) },
                Level       => 'user',
            },
            aexport => {
                description => gettext("Export autotimer to vdradmind.at"),
                short       => 'ax',
                callback    => sub{ $obj->autotimerexport(@_) },
                Level       => 'user',
            },
            atoogle => {
                description => gettext("Toogle Autotimer on or off 'aid'"),
                short       => 'at',
                callback    => sub{ $obj->autotimerToogle(@_) },
                Level       => 'user',
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $sql = "select count(*) from AUTOTIMER";
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    $console->message(sprintf(gettext("%d Autotimers exists."), $erg->[0][0]));
}


# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # file
    $self->{file} = $self->{config}->{file};

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

    # Interval to read timers and put to DB
    Event->timer(
        interval => $self->{interval},
        cb => sub{
            $self->autotimer();
        },
    );

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Old Version
    if(tableExists($obj->{dbh}, 'AUTOTIMER')) {
        # Nicht loeschen da das Userdata sind
        # Start
        $obj->{dbh}->do("alter table AUTOTIMER add Start char(4) default '0000'")
            unless(grep('Start', fields($obj->{dbh}, 'select * from AUTOTIMER')));
        # Stop
        $obj->{dbh}->do("alter table AUTOTIMER add Stop char(4) default '0000'")
            unless(grep('Stop', fields($obj->{dbh}, 'select * from AUTOTIMER')));
        # MinLength
        $obj->{dbh}->do("alter table AUTOTIMER add MinLength int(11) default '0'")
            unless(grep('MinLength', fields($obj->{dbh}, 'select * from AUTOTIMER')));
    }

    # Look for table or create this table
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS AUTOTIMER (
          Id int(11) auto_increment NOT NULL,
          Activ enum('y', 'n') default 'y',
          Done  enum('y', 'n') default 'y',
          Search varchar(100) NOT NULL default '',
          InFields set('title', 'subtitle', 'description' ) NOT NULL,
          Channels text default '',
          Start char(4) default '0000',
          Stop  char(4) default '0000',
          MinLength int(11) default '0',
          Priority int(11),
          Lifetime int(11),
          Dir varchar(100),
          PRIMARY KEY  (Id)
        )
    |);
}

# ------------------
sub autotimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $autotimerid = shift;

    # Get Autotimer
    my $sql = sprintf('select * from AUTOTIMER where Activ = "y" %s order by Id', ($autotimerid ? "AND Id = $autotimerid" : ''));
    my $att = $obj->{dbh}->selectall_hashref($sql, 'Id');

    # Get Timersmodule
    my $timermod = main::getModule('TIMERS');

    foreach my $id (sort keys %$att) {
        my $a = $att->{$id};

        if(ref $console) {
            $console->message(' ') if($console->{TYP} eq 'HTML');
            $console->message(sprintf(gettext("Look for AutotimerID(%d) with search '%s'"), $id, $a->{Search}));
        }

	my $search = buildsearch($a->{InFields}, $a->{Search});

    # Start and Stop
    if($a->{Start} and $a->{Stop}) {
        if($a->{Start} > $a->{Stop}) {
            $search .= "\n AND ((DATE_FORMAT(e.starttime, '%H%i') > $a->{Start} AND DATE_FORMAT(e.starttime, '%H%i') < 2359) OR (DATE_FORMAT(e.starttime, '%H%i') >= 0 and DATE_FORMAT(e.starttime, '%H%i') < $a->{Stop}))";
        } else {
            $search .= "\n AND (DATE_FORMAT(e.starttime, '%H%i') > $a->{Start} AND DATE_FORMAT(e.starttime, '%H%i') < $a->{Stop})";
        }
    }

    # Min Length
    if(exists $a->{MinLength} and $a->{MinLength}) {
        $search .= sprintf(" AND e.duration >= %d ", $a->{MinLength} * 60);
    }

    # Channels
    if($a->{Channels} and my @channelids = split(',', $a->{Channels})) {
        @channelids = map {$_ = "'$_'"} @channelids;
        $search = sprintf(' %s  AND channel_id in (%s)', $search, join(',', @channelids));
    }

    # Search for timer
    my $sql = qq|
SELECT
    e.eventid as eventid,
    e.channel_id as ChannelID,
    CONCAT_WS('~', e.title, e.subtitle) as File,
    e.description as Summary,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) - ( $timermod->{prevminutes} * 60 )), '%d') as Day,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) - ( $timermod->{prevminutes} * 60 )), '%H%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration + ( $timermod->{afterminutes} * 60 )), '%H%i') as Stop
FROM
    EPG as e,
    CHANNELS as c
WHERE
    ( $search )
    AND ( e.channel_id = c.Id )|;

# dumper $sql;
        my $data = $obj->{dbh}->selectall_hashref($sql, 'eventid') || next;

        if(ref $console) {
            $console->message(sprintf(gettext("Found %d Entrys for '%s' in EPG DB."), scalar keys %$data, $a->{Search}));
            map {
                $console->table($data->{$_});
            } sort keys %{$data}
        }

        # Every found and save this as timer
        my $c = 0;
        foreach my $Id (sort keys %$data) {
            $data->{$Id}->{Status} = 'y';
            $data->{$Id}->{Priority} = $a->{Priority};
            $data->{$Id}->{Lifetime} = $a->{Lifetime};

            my $file = $data->{$Id}->{File};
            $file =~ s/\'/\\\'/sg;

            # Check to double Shows ...
            my $sql = "select count(*) as cc from TIMERS where File like '%$file%'";
            my $ergebniss = $obj->{dbh}->selectrow_hashref($sql)->{cc};
            next if( $a->{Done} eq 'y' and $ergebniss );
            $data->{$Id}->{File} = sprintf('%s~%s', $a->{Dir}, $data->{$Id}->{File})
                if($a->{Dir});

            $c++;
            $console->message(sprintf(gettext("Save timer for '%s'"), $data->{$Id}->{File}))
                if(ref $console);
            my $erg = $timermod->saveTimer($data->{$Id});
        }
        $console->message(sprintf(gettext("Save %d timer for '%s'."), $c, $a->{Search}))
            if(ref $console);
    }
    $timermod->readData();

    if(ref $console) {
        $console->message(gettext("Autotimer update sucessfully."));
        $console->link({
            text => gettext("Back to autotimer list"),
            url => "?cmd=alist",
        }) if($console->typ eq 'HTML');
    }
}

# ------------------
sub autotimerCreate {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || 0;
    my $data    = shift || 0;

    $obj->autotimerEdit($watcher, $console, $timerid, $data);

}

# ------------------
sub autotimerEdit {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || 0;
    my $data    = shift || 0;

    my $mod = main::getModule('CHANNELS');
    my $modT = main::getModule('TIMERS');

    my $epg;
    if($timerid and not ref $data) {
        $epg = $obj->{dbh}->selectrow_hashref("select * from AUTOTIMER where Id = $timerid")
            or return $console->message(sprintf(gettext("Autotimer ID '%s' is not exist in Database!"),$timerid));

            # Channels Ids in Namen umwandeln
            my @channels = map { $_ = $mod->ChannelToName($_) } split(/[\s|,]+/, $epg->{Channels});
            $epg->{Channels} = \@channels;

            # question erwartet ein Array
            my @infields = split(/\s*,\s*/, $epg->{InFields});
            $epg->{InFields} = \@infields;

    }

    my $questions = [
        'Id' => {
            typ     => 'hidden',
            def     => $epg->{Id} || 0,
        },
        'Activ' => {
            typ     => 'confirm',
            def     => $epg->{Activ} || 'y',
            msg     => gettext('Switch this Autotimer on?'),
        },
        'Done' => {
            typ     => 'confirm',
            def     => $epg->{Done} || 'y',
            msg     => gettext('Register timer in DoneList (no double Tvshows)?'),
        },
        'Search' => {
            msg   => gettext("Search (minus will search with NOT this txt)\ni.e.: Enterprise,-Voyager"),
            req   => gettext('This is required!'),
            def   => $epg->{Search} || '',
        },
        'InFields' => {
            msg   => gettext('Search in this EPG fields'),
            typ   => 'checkbox',
            choices   => ['title', 'subtitle', 'description'],
            req   => gettext('This is required!'),
            def   => $epg->{InFields} || ['title','subtitle'],
            check   => sub{
                my $value = shift || return;
                my $data = shift || return error('No Data in CB');
                my @vals = (ref $value eq 'ARRAY') ? @$value : split(/\s*,\s*/, $value);

                foreach my $v (@vals) {
                    unless(grep($_ eq $v, @{$data->{choices}})) {
                        my $ch = join(' ', @{$data->{choices}});
                        return undef, sprintf(gettext("You can choose: %s!"),$ch);
                    }
                }
                return join(',', @vals);
            },
        },
        'Channels' => {
            typ     => 'list',
            def     => $epg->{Channels},
            choices => $mod->ChannelArray('Name'),
            options => 'multi',
            msg     => gettext('Restrict search to this channels'),
            check   => sub{
                my $value = shift || return;
                my @vals;
                foreach my $chname ((ref $value eq 'ARRAY' ? @$value : split(/\s*,\s*/, $value))) {
                    if( my $chid = $mod->NameToChannel($chname) || $mod->PosToChannel($chname)) {
                        push(@vals, $chid);
                    } else {
                        return undef, sprintf(gettext("This channel '%s' is not exists!"),$chname);
                    }
                }
                return join(',', @vals);
            },
        },
         'Start' => {
             typ     => 'string',
             def     => sub{
		             my $value = $epg->{Start} || return "";
                     $value = sprintf('%04d',$epg->{Start});
                     return substr($value, 0, 2) .':'. substr($value, 2, 2);
                 },
             msg     => gettext("Starttime in format 'HH:MM'"),
             check   => sub{
                 my $value = shift || return "";
        		 return "" if(not $value);
                 my @v = split(':', $value);
                 return undef, gettext('No right time!') if(scalar @v != 2);
                 $value = sprintf('%02d%02d',$v[0],$v[1]);
                 if(int($value) < 2400 and int($value) >= 0) {
                     return sprintf('%04d',$value);
                 } else {
                     return undef, gettext('No right time!');
                 }
             },
         },
         'Stop' => {
             typ     => 'string',
             def     => sub{
    		         my $value = $epg->{Stop} || return "";
                     $value = sprintf('%04d',$epg->{Stop});
                     return $value = substr($value, 0, 2) .':'. substr($value, 2, 2);
                 },
             msg     => gettext("Endtime in format 'HH:MM'"),
             check   => sub{
                 my $value = shift || return "";
        		 return "" if(not $value);
                 my @v = split(':', $value);
                 return undef, gettext('No right time!') if(scalar @v != 2);
                 $value = sprintf('%02d%02d',$v[0],$v[1]);
                 if(int($value) < 2400 and int($value) >= 0) {
                     return sprintf('%04d',$value);
                 } else {
                     return undef, gettext('No right time!');
                 }
             },
        },
        'MinLength' => {
            typ     => 'integer',
            msg     => gettext('Minimum length in minutes'),
            def     => $epg->{MinLength} || 0,
            check   => sub{
                my $value = shift || return;
                if($value > 0) {
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'Priority' => {
            typ     => 'integer',
            msg     => gettext('Priority (0 .. 99)'),
            def     => $epg->{Priority} || $modT->{Priority},
            check   => sub{
                my $value = shift || return;
                if($value > 0 and $value < 100) {
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'Lifetime' => {
            typ     => 'integer',
            msg     => gettext('Lifetime (0 .. 99)'),
            def     => $epg->{Lifetime} || $modT->{Lifetime},
            check   => sub{
                my $value = shift || return;
                if($value > 0 and $value < 100) {
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'Dir' => {
            msg     => gettext('Group all recording on a directory'),
            def     => $epg->{Dir},
        },
    ];

    # Ask Questions
    $data = $console->question(($timerid ? gettext('Edit Autotimer')
					 : gettext('New Autotimer')), $questions, $data);

    if(ref $data eq 'HASH') {
        delete $data->{Channel};

        # Last chance ;)
        return $console->message(gettext('No Search defined!'))
            unless($data->{Search});
    	$obj->insert($data);

    	$data->{Id} = $obj->{dbh}->selectall_arrayref('SELECT max(ID) FROM AUTOTIMER')->[0][0]
    		if(not $data->{Id});

        $console->message(gettext('Autotimer saved!'));

        return $obj->autotimer($watcher, $console, $data->{Id} );
    }
    return 1;
}

# ------------------
sub autotimerDelete {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || return $console->message(gettext("No AutotimerID to delete! Please use adelete 'aid'"));   # If timerid the edittimer

    my $erg = $obj->{dbh}->do("delete from AUTOTIMER where Id in ($timerid)")
        or return $console->message(sprintf gettext("AutotimerID '%s' is not exist in Database!"), $timerid);
    $console->message(sprintf gettext("Autotimer %s is deleted."), $timerid);
}

# ------------------
sub autotimerToogle {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || return $console->message(gettext("No AutotimerID to toogle! Please use atoogle 'aid'"));   # If timerid the edittimer

    my $status = $obj->{dbh}->selectrow_arrayref("select Activ from AUTOTIMER where Id = $timerid")->[0]
        or return error("This AutotimerID: '$timerid' is not exist in Database!");

    my $newstatus = ($status eq 'n') ? 'y' : 'n';
    my $erg = $obj->{dbh}->do("UPDATE AUTOTIMER set Activ = '$newstatus' where Id = $timerid")
        or return $console->message(sprintf(gettext("AutotimerID '%s' is not exist in Database!"),$timerid));
    my $text = ($newstatus eq 'n') ? gettext('disabled')
                                   : gettext('activated');

    $console->message(sprintf gettext("Autotimer %s is %s."), $timerid, $text);
}

# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || '';

    my $sql = "select Id, Activ as Act, Search, Channels, Dir, Start, Stop, MinLength as Min from AUTOTIMER where AUTOTIMER.Id like '%$id%' order by AUTOTIMER.Id";
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    my $mod = main::getModule('CHANNELS');

    $console->table($erg, {channels => $mod->ChannelHash('Id')});
}

# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;

    if(ref $data eq 'HASH') {
        my ($names, $vals, $kenn);
        map {
            push(@$names, $_);
            push(@$vals, $data->{$_}),
            push(@$kenn, '?'),
        } sort keys %$data;

        my $sql = sprintf("REPLACE INTO AUTOTIMER (%s) VALUES (%s)",
                join(', ', @$names),
                join(', ', @$kenn),
        );
        my $sth = $obj->{dbh}->prepare( $sql );
        $sth->execute( @$vals );
    } else {
        my $sth = $obj->{dbh}->prepare('REPLACE INTO AUTOTIMER VALUES (?,?,?,?,?,?,?,?,?)');
        $sth->execute( @$data );
    }
}

sub autotimerimport {
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $AT_FILENAME = shift || return $console->message(gettext("No filename to import! Please use aimport '../vdradmind.at'"));
    my $mod = main::getModule ('CHANNELS');
    my $cnt = "0";
    open(AT_FILE, "$AT_FILENAME") || return $console->message(sprintf(gettext("can't open file '%s' : %s"),$AT_FILENAME,$!));
    while(<AT_FILE>) {
        chomp;
        next if ($_ eq "");

        my($active, $pattern, $section, $start, $stop, $episode, $prio,
            $lft, $channel, $directoy) = split(/\:/, $_);

        my $data;
        $data->{Id} = 0;
        if($active eq "1") { $data->{Activ} = 'y'; }
	    else               { $data->{Activ} = 'n'; }

        my @vals;
        $data->{Done} = 'y';
        $data->{Search} = $pattern;
        push(@vals, "title")
            if($section & 1);
        push(@vals, "subtitle")
            if($section & 2);
        push(@vals, "description")
            if($section & 4);
        $data->{InFields} = join(',', @vals);
        if( $channel ne "") {
            if (my $chid = $mod->PosToChannel($channel)) {
                $data->{Channels} = $chid;
            }
        }
        $data->{Priority} = $prio;
        $data->{Lifetime} = $lft;
        $data->{Dir} = $directoy;
        $data->{Start} = $start;
        $data->{Stop} = $stop;
#       $data->{Episode} = $episode; # xxv don't support episode, i belief "1" means File = title~subtitle / "0" means File = title

        $obj->insert($data);
        ++$cnt;
    }
    close (AT_FILE);
    return $console->message(sprintf(gettext("%d Autotimer read from file %s."),$cnt,$AT_FILENAME));
}

sub autotimerexport {
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $AT_FILENAME = shift || return $console->message(gettext("No filename to export! Please use aexport '../vdradmind.at'"));
    my $id      = shift || '';

    my $mod = main::getModule ('CHANNELS');

    my $sql = "select Activ, Search, InFields, Channels, Dir, Start, Stop, Priority, Lifetime from AUTOTIMER where AUTOTIMER.Id like '%$id%' order by AUTOTIMER.Id";
    my $atlist = $obj->{dbh}->selectall_arrayref($sql);
    my $cnt = "0";
    open(AT_FILE, ">$AT_FILENAME") || return $console->message(sprintf(gettext("can't open file '%s' : %s"),$AT_FILENAME,$!));
    foreach my $data (@$atlist) {
        if($data->[1] ne "" and $data->[2] ne "") {

        my $active = "0";
        my $pattern = $data->[1];
        my $section = "0";
        my $start = "";
        my $stop = "";
        my $episode = "0"; # xxv don't support episode, i belief "1" means File = title~subtitle / "0" means File = title
        my $prio = "";
        my $lft = "";
        my $channel = "";
        my $directoy = "";

        $active = "1" if($data->[0] eq "y");

        $section |= 1 if ($data->[2]=~/^.*title.*$/);
        $section |= 2 if ($data->[2]=~/^.*subtitle.*$/);
        $section |= 4 if ($data->[2]=~/^.*description.*$/);

        if( $data->[3]) {
            foreach my $ch (@{$mod->ChannelArray ('Id')}) {
                if ($data->[3] eq $ch->[0]) { # vdradmin suppport only one Channel
                    $channel = $ch->[1];
                    last;
                }
            }
        }

        $directoy = $data->[4] if($data->[4]);
        $start = $data->[5] if($data->[5] and $data->[5] ne $data->[6]); # Exclude if start and stop if same value e.g. 0000
        $stop = $data->[6] if($data->[6] and $data->[5] ne $data->[6]);
        $prio = $data->[7] if($data->[7]);
        $lft = $data->[8] if($data->[8]);

        print AT_FILE sprintf("%s:%s:%s:%s:%s:%s:%s:%s:%s:%s\n",
            $active,$pattern,$section,$start,$stop,$episode,$prio,$lft,$channel,$directoy)
            || return return $console->message(sprintf(gettext("can't write file '%s' : %s"),$AT_FILENAME,$!));
        ++$cnt;
        }
    }
    close (AT_FILE);
    return $console->message(sprintf(gettext("%d Autotimer write to file %s"),$cnt,$AT_FILENAME));
}

1;
