package XXV::MODULES::EPG;
use strict;

use Tools;
use Event::File;
use Date::Manip;
use Locale::gettext;


# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'EPG',
        Description => gettext('This module want read new EPG Data and save this in the Database.'),
        Version => '0.01',
        Date => '18.08.2004',
        Author => 'xpix',
        Order => 50,
        Preferences => {
            epgfile => {
                description => gettext('Where is the epg.data file on youre system'),
                default     => '/video/epg.data',
                type        => 'file',
                required    => gettext('This is required!'),
            },
            interval => {
                description => gettext('The interval to parse new EPG to DB in sec'),
                default     => 300,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            restart => {
                description => gettext('The interval in houres(!) to restart the parse Epg Data'),
                default     => 3,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            timers => {
                description => gettext('The timers where you look an running at (comma separated list)'),
                default     => '12:00,18:00,20:15,22:00',
                type        => 'string',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            search => {
                description => gettext('Search in EPG Data'),
                short       => 's',
                callback    => sub{ $obj->search(@_) },
            },
            program => {
                description => gettext("List program for channel 'channel name'"),
                short       => 'p',
                callback    => sub{ $obj->program(@_) },
            },
            display => {
                description => gettext("Show program event 'eventid'"),
                short       => 'd',
                callback    => sub{ $obj->display(@_) },
            },
            estatus => {
                description => gettext('Status from EPG Data'),
                short       => 'et',
                callback    => sub{ $obj->status(@_) },
            },
            now => {
                description => gettext('Display events is running now'),
                short       => 'n',
                callback    => sub{ $obj->runningNow(@_) },
            },
            epurge => {
                description => gettext('Delete all EPG Events and restart the Scan process'),
                short       => 'ep',
                callback    => sub{ $obj->epurge(@_) },
                Level       => 'admin',
            },
            erestart => {
                description => gettext('Restart the EPG scan process'),
                short       => 'er',
                callback    => sub{
                    my $watcher = shift || return error ('No Watcher!');
                    my $console = shift || return error ('No Console');
                    delete $obj->{attr};
                    $obj->{EventTail}->start;
                    return $console->message(gettext('EPG Scan restartet!'));
                },
                Level       => 'admin',
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $sql = "select count(*) from EPG";
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    $console->message(sprintf('%d rows in EPG table.', $erg->[0][0]));
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->init or return error('Problem to initialze Module');

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Look for table or create this table
    foreach my $table (qw/EPG OLDEPG/) {
        $obj->{dbh}->do(qq|
          CREATE TABLE IF NOT EXISTS $table (
              eventid int(11) NOT NULL default '0',
              title varchar(100) NOT NULL default '',
              subtitle varchar(100) default '',
              description longtext,
              channel_id varchar(100) NOT NULL default '',
              starttime datetime NOT NULL default '0000-00-00 00:00:00',
              duration int(11) NOT NULL default '0',
              tableid varchar(100) default '',
              addtime datetime NOT NULL default '0000-00-00 00:00:00',
              PRIMARY KEY (eventid),
              INDEX (starttime),
              INDEX (channel_id)
            )
        |);
    }

    return 1, warn ('No EPG File found') if( ! -e  $obj->{epgfile});

    $obj->{EventTail} = $obj->startTail( $obj->{epgfile} );

    # Restart watcher every x houres
    Event->timer(
        interval => $obj->{restart}*3600,
        cb => sub{
            delete $obj->{attr};
            $obj->{EventTail}->start;
        },
    );

    # Restart epurge every 6 houres
    Event->timer(
        interval => 6*3600,
        cb => sub{
            $obj->epurge;
        },
    );

    # Delete the old EPG Entrys in Interval
    Event->timer(
        interval => $obj->{interval},
        cb => sub{ $obj->repairData },
    );

    # Repair now the Data ...
    $obj->repairData;

    return 1;
}

# ------------------
sub epurge {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    $obj->{EventTail}->cancel;
    $obj->{dbh}->do('delete from EPG');
    $obj->{EventTail} = $obj->startTail( $obj->{epgfile} );
    return $console->message(gettext('EPG Scan completely restartet!'))
        if(ref $console);
}

# ------------------
sub startTail {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $file = shift || return error('No File');

    $obj->{OLDPOS} = 0;

    # Read new EPG Data if epg.data touched
    my $event = Event::File->tail(
        cb => sub {
            $obj->readData(@_);
        },
        file => $file,
        postion => 0,
    );

    return $event;
}

# ------------------
sub repairData {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Copy and delete old EPG Entrys
    $obj->{dbh}->do('REPLACE INTO OLDEPG SELECT * FROM EPG WHERE (UNIX_TIMESTAMP(EPG.starttime) + EPG.duration) < UNIX_TIMESTAMP()');
    $obj->{dbh}->do('DELETE FROM EPG WHERE (UNIX_TIMESTAMP(EPG.starttime) + EPG.duration) < UNIX_TIMESTAMP()');

    # Delete double EPG Entrys
    my $erg = $obj->{dbh}->selectall_arrayref('SELECT eventid FROM EPG GROUP BY starttime, channel_id having count(*) > 1');
    foreach my $row (@$erg) {
        $obj->{dbh}->do(sprintf("DELETE FROM EPG WHERE eventid = '%d'", $row->[0]));
    }
}

# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!');
    my $attr = shift || return;

    $attr->{title} = gettext("No Title")
        unless($attr->{title});
    $attr->{description} = ""
        unless($attr->{description});

    # look for NID-TID-SID for unique eventids (SID 0-30000 / TID 0 - 1000 / NID 0 - 10000
    my @id = split('-', $attr->{channel});
    my $eventid = sprintf('%d%d', ($id[-3] + $id[-2] + $id[-1])%10000 , $attr->{eventid});

    my $sth = $obj->{dbh}->prepare('REPLACE INTO EPG(eventid, title, subtitle, description, channel_id, duration, tableid, starttime, addtime) VALUES (?,?,?,?,?,?,?,FROM_UNIXTIME(?),FROM_UNIXTIME(?))');
    $sth->execute(
        $eventid,
        $attr->{title},
        $attr->{subtitle},
        $attr->{description},
        $attr->{channel},
        $attr->{duration},
        $attr->{tableid},
        $attr->{starttime},
        time,
    ) if($attr->{channel});
}

# ------------------
sub readData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!' );
    my $line = shift || return error ('No Line!' );

    # !!! OldPosition: %s is higher as actual position: %d
    if($watcher->position < $obj->{OLDPOS}) {
        return undef;
    }

    # Ok, save ...
    if($line eq 'e' and $obj->{attr}->{eventid}) {
        $obj->insert($obj->{attr});
        my $channel = $obj->{attr}->{channel};
        $obj->{attr} = {};
        $obj->{attr}->{channel} = $channel;
    }
    my ($mark, $data) = $line =~ /(\S)\s+(.+)/g;
    return unless($mark and $data);

    # New Channelname
    if($mark eq 'C') {
        $obj->{attr}->{channel} = (split('\s', $data))[0];
    } elsif($mark eq 'E') {
        ($obj->{attr}->{eventid}, $obj->{attr}->{starttime}, $obj->{attr}->{duration}, $obj->{attr}->{tableid}) = split('\s', $data);
    } elsif($mark eq 'T') {
        $obj->{attr}->{title} = $data;
    } elsif($mark eq 'S') {
        $obj->{attr}->{subtitle} = $data;
    } elsif($mark eq 'D') {
        $obj->{attr}->{description} = $data;
    }
    $obj->{OLDPOS} = $watcher->position;
}

# ------------------
sub search {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text = shift || return $console->message(gettext("No Text to search! Please use search 'text'"));

    my $search = buildsearch("e.title,e.subtitle,e.description",$text);
    my $sql = qq|
select
    e.eventid as Service,
    e.title as Title,
    e.subtitle as __Subtitle,
    c.Name as Channel,
    c.POS as __Pos,
    DATE_FORMAT(e.starttime, '%H:%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%H:%i') as Stop,
    DATE_FORMAT(e.starttime, '%a %d.%m') as Day,
    e.description
from
    EPG as e, CHANNELS as c
where
    e.channel_id = c.Id
    AND ( $search )
order by
    starttime
|;
#dumper($sql);
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    $console->table($erg);
}

# ------------------
sub program {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $channel = shift || return $console->message(gettext("No Channel to list the Programm! Please use program 'channel'"));

    my $mod = main::getModule('CHANNELS');
    my $cid;
    if($channel =~ /^\d+$/sig) {
        $cid = $mod->PosToChannel($channel)
            or return $console->message(sprintf(gettext("This channel '%s' is not exist in Database!"),$channel));
    } else {
        $cid = $mod->NameToChannel($channel)
            or return $console->message(sprintf(gettext("This channel '%s' is not exist in Database!"),$channel));
    }

    my $sql = qq|
select
    EPG.eventid as Service,
    EPG.title as Title,
    EPG.subtitle as __Subtitle,
    DATE_FORMAT(starttime, '%H:%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + duration), '%H:%i') as Stop,
    DATE_FORMAT(starttime, '%a %d.%m') as Day
from
    EPG, CHANNELS
where
    EPG.channel_id = CHANNELS.Id and
    channel_id = '$cid'
order by
    starttime
|;
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    $console->table($erg, {channels => $mod->ChannelArray('Name')});
}

# ------------------
sub display {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $eventid = shift || return $console->message(gettext("No EventID to display the Event Programm! Please use display 'eventid'"));

    my $sql = qq|
select
    e.eventid as Id,
    e.title as Title,
    e.subtitle as Subtitle,
    DATE_FORMAT(e.starttime, '%W: %H:%i %e.%c.%Y') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%W: %H:%i %e.%c.%Y') as Stop,
    c.Name as Channel,
    e.description as Description,
    (unix_timestamp(e.starttime) + e.duration - unix_timestamp())/duration*100 as Percent
from
    EPG as e,CHANNELS as c
where
    e.channel_id = c.Id
    and eventid = '$eventid'
|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    ######################
    # Lookup failed, so we look at oldepg
    if(scalar @{$erg} == 0 ) {
    $sql = qq|
select
    e.eventid as Id,
    e.title as Title,
    e.subtitle as Subtitle,
    DATE_FORMAT(e.starttime, '%W: %H:%i %e.%c.%Y') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%W: %H:%i %e.%c.%Y') as Stop,
    c.Name as Channel,
    e.description as Description,
    999 as Percent
from
    OLDEPG as e,CHANNELS as c
where
    e.channel_id = c.Id
    and eventid = '$eventid'
|;
    $erg = $obj->{dbh}->selectall_arrayref($sql);
}
    ######################
    unshift(@$erg, $fields);

    $console->table($erg);
}

# ------------------
sub runningNow {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $zeit   = UnixDate(ParseDate(shift),"%s") || time;

    $zeit += 86400 if($zeit < time);

    $zeit++;

    my $sql =
qq|
select
    EPG.eventid as Service,
    EPG.title as Title,
    EPG.subtitle as __Subtitle,
    CHANNELS.Name as Channel,
    CHANNELS.POS as __Channel,
    DATE_FORMAT(EPG.starttime, "%H:%i") as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + duration), "%H:%i") as Stop,
    (unix_timestamp(EPG.starttime) + EPG.duration - unix_timestamp())/duration*100 as Percent
FROM
    EPG, CHANNELS
WHERE
    EPG.channel_id = CHANNELS.Id
    AND $zeit BETWEEN UNIX_TIMESTAMP(EPG.starttime)
    AND (UNIX_TIMESTAMP(EPG.starttime) + EPG.duration)
ORDER BY
    CHANNELS.POS|;
#dumper($sql);
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    $console->table($erg,{timers => $obj->{timers}});
}

# ------------------
sub getId {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $id = shift || return error ('No Id!' );
    my $erg = $obj->{dbh}->selectrow_hashref(sprintf('select * from EPG WHERE eventid = %d', $id), 'eventid')
        or return error("No Event to load");
    return $erg;
}

1;
