package XXV::MODULES::GRAB;
use strict;

use Tools;
use Event::File;
use Locale::gettext;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'GRAB',
        Description => gettext('This module grab a picture from livestream.'),
        Version => '0.01',
        Date => '06.09.2004',
        Author => 'xpix',
        Preferences => {
            xsize => {
                description => gettext('The x size from picture'),
                default     => 320,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            ysize => {
                description => gettext('The y size from picture'),
                default     => 240,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            file => {
                description => gettext('The place to save the image file'),
                default     => '/tmp/live.jpg',
                type        => 'file',
                required    => gettext('This is required!'),
            },
            interval => {
                description => gettext('The interval to grab the image'),
                default     => 300,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            grab => {
                description => gettext('Grab a picture'),
                short       => 'gr',
                callback    => sub{ $obj->grab(@_) },
            },
            gdisplay => {
                description => gettext('Display the picture'),
                short       => 'gd',
                callback    => sub{ $obj->display(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # i need the svdrp module
    $self->{svdrp} = main::getModule('SVDRP');

    # Interval to read timers and put to DB
    Event->timer(
        interval => $self->{interval},
        cb => sub{
            $self->grab();
        },
    );

	return $self;
}

# ------------------
sub grab {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;

    # the svdrp module
    my $svdrp = $obj->{svdrp};

    # the command
    my $cmd = sprintf('grab %s jpeg 80 %d %d',
            $obj->{file},
            $obj->{xsize},
            $obj->{ysize},
    );

    my $erg = $svdrp->command($cmd);
    $console->message($erg)
        if(ref $console);
    return 1;
}

# ------------------
sub display {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $file = shift || $obj->{file} || return error ('No File to display');

    $obj->grab();

    return $console->image($file);
}
1;
