package XXV::MODULES::INTERFACE;

use Locale::gettext;
use XXV::OUTPUT::Dump;
use IO::Socket::INET;
use Tools;

use Data::Dumper; #XXX

use strict;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'INTERFACE',
        Description => gettext('This module is a multichannel telnet server with dumped output for second party software.'),
        Version => '0.01',
        Date => '06.09.2004',
        Author => 'xpix',
        Preferences => {
            Clients => {
                description => gettext('Max allowed Connection in the same time'),
                default     => 5,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            Port => {
                description => gettext('Portnumber to connect the client'),
                default     => 8002,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->init or return error('Problem to initialze Module');

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # globals
    my $channels;

    my $prompt = "XXV> ";

    # make socket
	my $socket = IO::Socket::INET->new(
		Listen		=> $obj->{Clients} || 5,
		LocalPort	=> $obj->{Port}    || 8002,
		Reuse		=> 1);
	warn sprintf("[Fatal] Cannot build initial socket for %s on port %d. [%s]\n",
	        $obj->{Name}, $obj->{Port}, $!) unless $socket;

    # install an initial watcher
    Event->io(
        fd => $socket,
        cb => sub {
                # make "channel" number
                my $channel=++$channels;

                # accept client
                my $client=$socket->accept;
                warn "[Error] Cannot connect interface to new client.\n" and return unless $client;
                $client->autoflush;

                # XXX: Dieser Login wird erstmal emuliert ;)
                # Eine Lsung muss ein, das sich der TelnetClient einloggt (login xxv xxv)
                my $userMod = main::getModule('USER');
                my $user = {};
                unless(exists $obj->{USER} or $userMod->{Active} ne 'y') {
                    # Login
                    $user = $userMod->check('xxv', 'xxv');
                }

                # install a communicator
                Event->io(
                    fd => $client,
                    cb => sub {
                            my $watcher = shift;

                            # read new line and report it
                            my $handle=$watcher->w->fd;

                            my $line=<$handle>;
                           unless($line) {
                                $watcher->w->cancel;
                                $handle->close();
                                undef $watcher;
                                return 1;
                            }
                            $line =~ s/[\r|\n]//sig
                                if(defined $line);

                            my $console = XXV::OUTPUT::Dump->new(
                                -handle => $handle,
                                -dbh    => $obj->{dbh},
                            );

                            $obj->handleInput($watcher, $console, $line, $user);
                        },
                );

                # welcome
                $client->print(sprintf(gettext("Welcome to XXV System Ver.: %s.\r\nThis is session %s.\r\n---\r\nEOF\r\n"),
                    $obj->{MOD}->{Version}, $channel));
            },
    );
}

# ------------------
sub handleInput {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $telnet = main::getModule('TELNET');
    my $ret = $telnet->handleInput(@_);
}

1;
