package XXV::MODULES::MUSIC;
use strict;

use Tools;
use MP3::Icecast;
use MP3::Info;
use CGI;
use Locale::gettext;

$SIG{CHLD} = 'IGNORE';

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'MUSIC',
        Description => gettext('This module managed music files.'),
        Version => '0.01',
        Date => '06.09.2004',
        Author => 'xpix',
        Preferences => {
            path => {
                description => gettext('The path to mp3 directory'),
                default     => '/music/MP3',
                type        => 'dir',
                required    => gettext('This is required!'),
            },
            port => {
                description => gettext('Port to listen for icecast clients.'),
                default     => 8100,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            clients => {
                description => gettext('Maximum Clients to connect at the same time.'),
                default     => 5,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            mrefresh => {
                description => gettext('Reread the music tree.'),
                short       => 'mr',
                callback    => sub{ $obj->refresh(@_) },
            },
            mplay => {
                description => gettext("play the file 'fid'"),
                short       => 'mp',
                callback    => sub{ $obj->play(@_) },
            },
            mplaylist => {
                description => gettext("get a m3u playlist for 'fid'"),
                short       => 'm3',
                callback    => sub{ $obj->playlist(@_) },
            },
            mlist => {
                description => gettext("list music 'dir'"),
                short       => 'ml',
                callback    => sub{ $obj->list(@_) },
            },
            mgroups => {
                description => gettext('list music groups'),
                short       => 'mg',
                callback    => sub{ $obj->groups(@_) },
            },
            msearch => {
                description => gettext("search music 'txt'"),
                short       => 'mf',
                callback    => sub{ $obj->search(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

    # Read File to Database, if the DB empty
    $self->refresh()
        unless($self->{dbh}->selectall_arrayref("select count(*) from MUSIC")->[0][0]);

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    #create an instance to find all files below /usr/local/mp3
    $obj->{ICE} = MP3::Icecast->new();
    $obj->{ICE}->recursive(1);
    $obj->{ICE}->add_directory($obj->{path});

    $obj->{SOCK} = IO::Socket::INET->new(
        LocalPort => $obj->{port}, #standard Icecast port
        Listen    => $obj->{clients},
        Proto     => 'tcp',
        Reuse     => 1,
        Timeout   => 3600
    );

    my $channels;

    Event->io(
        fd => $obj->{SOCK},
        cb => sub {
            # accept client
            my $client = $obj->{SOCK}->accept;
            warn "[Error] Cannot connect to new icecast client.\n" and return unless $client;
            $client->autoflush;

            # make "channel" number
            my $channel=++$channels;

            # install a communicator
            Event->io(
                fd => $client,
                poll => 'r',
                cb => sub {
                    my $watcher = shift;
                    # report
                    warn "[Server] Talking on icecast channel $channel.\n";

                    # read new line and report it
                    my $handle=$watcher->w->fd;
                    my $Req = getFromSocket($handle);
                    my $data = $obj->parseRequest($Req);
                    my $files = $obj->handleInput($data);
                    unless(ref $files eq 'ARRAY') {
                        $watcher->w->cancel;
                        $client->close();
                        undef $watcher;
                        return 1;
                    }

                    $obj->stream($files, $client);

                    $watcher->w->cancel;
                    undef $watcher;
                    $client->close;
                },
            );

            # report
            warn "[Server] Opened new icecast channel $channel.\n";
        },
    );


    # Look for table or create this table
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS MUSIC (
          ID varchar(128) NOT NULL,
          FILE text NOT NULL,
          ARTIST varchar(128) default 'unknown',
          ALBUM varchar(128) default 'unknown',
          TITLE varchar(128) default 'unknown',
          COMMENT varchar(128),
          TRACKNUM varchar(10) default '0',
          YEAR int(4) default 1970,
          GENRE varchar(128),
          BITRATE int(4),
          FREQUENCY varchar(4),
          SECS int (11) NOT NULL,
          PRIMARY KEY  (ID)
        )
    |);

    $obj->{fields} = fields($obj->{dbh}, 'select * from MUSIC');

}

# ------------------
sub refresh {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;

    $obj->{CACHE} = {};

    $obj->{dbh}->do( 'DELETE FROM MUSIC' );

    my @files = $obj->{ICE}->files;
    foreach my $file (@files) {
        my $info = MP3::Info->new($file);
        $obj->insert($info);
    }
    $console->message(scalar @files . gettext(' music files in database saved!'))
        if(ref $console);
}

# ------------------
sub play {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data = shift || return error ('No data');

    $console->player();
}

# ------------------
sub playlist {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data = shift || return error ('No data');

    my $host = main::getModule('STATUS')->IP;
    my $output = "#EXTM3U\r\n";

    foreach my $id (split('_', $data)) {
        my $data = $obj->{dbh}->selectrow_hashref("select * from MUSIC where ID = '$id'");
        my $file = sprintf('http://%s:%d/?cmd=play&data=%s&field=ID', $host, $obj->{port}, $id);
        $output .= sprintf("#EXTINF:%d,%s - %s (%s)\r\n",$data->{SECS},$data->{TITLE},$data->{ARTIST},$data->{ALBUM});
        $output .= sprintf("%s\r\n", $file);
    }
    $console->{handle}->print($output);
}

# ------------------
sub groups {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $erg;
    push(@$erg, [gettext('Genres'), $obj->groupfields('GENRE')]);
    push(@$erg, [gettext('Artists'), $obj->groupfields('ARTIST')]);
    push(@$erg, [gettext('Albums'), $obj->groupfields('ALBUM')]);

    my $sql = "select ID, GENRE, ARTIST, ALBUM, FILE from MUSIC";
    my $param = $obj->{dbh}->selectall_hashref($sql, 'FILE');

    $console->groups($erg, {rows => $param, fields => ['GENRE', 'ARTIST', 'ALBUM']});
}

# ------------------
sub search {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text   = shift || return $console->message(gettext("No Text to search! Please use msearch 'text'"));

	my $search = buildsearch("ALBUM,ARTIST,TITLE",$text);
    my $sql = qq|
SELECT
	ID,
	ARTIST AS __ARTIST,
	ALBUM as __ALBUM,
	TITLE,
	TRACKNUM,
	YEAR as __YEAR,
	DATE_FORMAT(FROM_UNIXTIME(SECS), '%i:%s') as Time,
	GENRE as __GENRE,
	COMMENT as __COMMENT
FROM
	MUSIC
WHERE
	( $search )
ORDER BY
	FILE
|;
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg);
}

# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $album   = shift || $obj->{dbh}->selectall_arrayref("select ALBUM from MUSIC limit 1")->[0][0];

    my $sql = "select ID, ARTIST AS __ARTIST, ALBUM as __ALBUM, TITLE, TRACKNUM, YEAR as __YEAR, DATE_FORMAT(FROM_UNIXTIME(SECS), '%i:%s') as Time, GENRE as __GENRE, COMMENT as __COMMENT from MUSIC where ALBUM like '%$album%' order by FILE";
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg);
}

# ------------------
sub handleInput {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $data    = shift || return error ('No Request!' );
    my $cgi = CGI->new( $data->{Query} );

    my $ucmd = $cgi->param('cmd')   || 'play';
    my $ufield = $cgi->param('field') || 'ID';
    my $udata = $cgi->param('data') || '*';

    my $files;
    if($ucmd eq 'play' and $ufield and my @search = split(',',$udata)) {
        $files = $obj->field2path($ufield, \@search);
    } else {
        dumper([$ucmd, $ufield, $udata]);
        return error "I don't understand this command '$ucmd'";
    }
    return $files;
}

# ------------------
sub field2path {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $field = shift || return error ('No Field!' );
    my $data = shift || return error ('No ids!' );

    map {$_ = $obj->{dbh}->quote($_)} @$data;
    my $sql = sprintf "select FILE, %s from MUSIC", $field;
    $sql .= sprintf " where %s in (%s)", $field, join(',', @$data)
        if($data->[0] ne '*');
    my $ret = $obj->{dbh}->selectall_hashref($sql, 'FILE');
    my @files = sort keys %$ret;
    return \@files;
}

# ------------------
sub groupfields {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $field = shift || return error ('No Field!' );
    return $obj->{CACHE}->{GROUPS}->{$field}
        if(exists $obj->{CACHE}->{GROUPS}->{$field});

    my $sql = sprintf "select %s from MUSIC group by %s", $field, $field;
    my $ret = $obj->{dbh}->selectall_hashref($sql, $field);
    my @names = sort keys %$ret;
    @{$obj->{CACHE}->{GROUPS}->{$field}} = @names;

    return \@names;
}


# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;

    my @setdata;
    foreach my $name (keys %$data) {
        next unless(grep($name eq $_, @{$obj->{fields}}));
        push(@setdata, sprintf("%s=%s", $name, $obj->{dbh}->quote($data->{$name})));
    }

    # MD5(File) as ID
    push(@setdata, sprintf("ID = MD5(%s)", $obj->{dbh}->quote($data->{FILE})));
    my $sql = sprintf('INSERT INTO MUSIC SET %s', join(', ', @setdata));
    $obj->{dbh}->do( $sql );
}

# ------------------
sub stream {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $files = shift || return error ('No Files!' );
    my $client = shift || return error ('No Client!' );


    defined(my $child = fork()) or die "Can't fork: $!";
    if($child == 0) {
        $obj->{SOCK}->close;
        $obj->{dbh}->{InactiveDestroy} = 1;

        while(@$files) {
            my $file = shift @$files;
            my $erg = $obj->{ICE}->stream($file,0,$client)
                || last;
        }
        exit 0;
    }
}

# ------------------
sub parseRequest {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $Req = shift || return error ('No Request!' );

	if($Req->[0] =~ /^GET (\/[\w\.\/-\:]*)([\?[\w=&\.\+\%-\:\!]*]*)[\#\d ]+HTTP\/1.\d$/) {
        my $data = {};
		($data->{Request}, $data->{Query}) = ($1, $2 ? substr($2, 1, length($2)) : undef);

    	# parse header
    	foreach my $line (@$Req) {
    		if($line =~ /Referer: (.*)/) {
    			$data->{Referer} = $1;
    		}
    		if($line =~ /Host: (.*)/) {
    			$data->{HOST} = $1;
    		}
    		if($line =~ /Authorization: basic (.*)/i) {
    			($data->{username}, $data->{password}) = split(":", MIME::Base64::decode_base64($1), 2);
    		}
    		if($line =~ /User-Agent: (.*)/i) {
    			$data->{http_useragent} = $1;
    		}
    	}
        return $data;
	} else {
        warn "Unknown Request: <%s>\n", join("\n", @$Req);
		return;
	}
}


1;
