package XXV::MODULES::REMOTE;

use strict;

use Tools;
use Locale::gettext;
# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'REMOTE',
        Description => gettext('This module emulate a remote control.'),
        Version => '0.01',
        Date => '10.10.2004',
        Author => 'xpix',
        Level => 'user',
        Commands => {
            remote => {
                description => gettext("Display ir remote 'cmd'"),
                short       => 'r',
                callback    => sub{ $obj->remote(@_) },
            },
            switch => {
                description => gettext("Switch to channel 'cid'"),
                short       => 'sw',
                callback    => sub{ $obj->switch(@_) },
                Level       => 'user',
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # i need the svdrp module
    $self->{svdrp} = main::getModule('SVDRP');

	return $self;
}

# ------------------
sub remote {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $command = shift;

    unless($command) {
        my $mod = main::getModule('GRAB');
        my $params = {
            width => $mod->{xsize},
            height => $mod->{ysize},
        };
        return $console->remote(undef, $params);
    } else {
        # the svdrp module
        my $svdrp = $obj->{svdrp};

        my $translate = {
            '<' => 'Channel-',
            '>' => 'Channel+',
            '+' => 'Volume+',
            '-' => 'Volume-',
            '>>' => 'FastFwd',
            '<<' => 'FastRew',
            'VolumePlus' => 'Volume+',
            'VolumeMinus' => 'Volume-',
            'Null' => '0',
        };

        $command = $translate->{$command}
            if(exists $translate->{$command});

        # the command
        my $cmd = sprintf('hitk %s', $command);
        my $erg = $svdrp->command($cmd);

        $console->message($erg)
            if(ref $console);
    }
    return 1;
}

# ------------------
sub switch {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $channel = shift || return $console->message(gettext("Sorry, but i need a channel to switch!"));

    # the svdrp module
    my $svdrp = $obj->{svdrp};

    # the command
    my $cmd = sprintf('chan %s', $channel);
    my $erg = $svdrp->command($cmd);

    $console->message($erg)
        if(ref $console);
    return 1;
}

1;
