package XXV::MODULES::SVDRP;

use IO::Socket::INET;
use Tools;
use Locale::gettext;
use strict;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'SVDRP',
        Description => gettext('This module a telnet client for sdvrp.'),
        Version => '0.01',
        Date => '07.09.2004',
        Author => 'xpix',
        Preferences => {
            VdrHost => {
                description => gettext('Hostname from vdr'),
                default     => 'localhost',
                type        => 'string',
                required    => gettext('This is required!'),
            },
            VdrPort => {
                description => gettext('Portnumber from svdrp in vdr'),
                default     => 2001,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            sstatus => {
                description => gettext('Status from svdrp'),
                short       => 'ss',
                callback    => sub{ $obj->status(@_) },
                Level       => 'user',
            },
            scommand => {
                description => gettext('Send a command to svdrp'),
                short       => 'sc',
                callback    => sub{ $obj->scommand(@_) },
                Level       => 'admin',
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

	return $self;
}

# ------------------
sub connect {
# ------------------
    my $obj = shift || return error ('No Object!' );
	my $SOCKET = IO::Socket::INET->new(
		PeerAddr => $obj->{VdrHost},
		PeerPort => $obj->{VdrPort},
		Proto => 'tcp',
        Type => SOCK_STREAM,
        Timeout => 1,
	) or return error($@);
    return $SOCKET;
}

# ------------------
sub command {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $cmd = shift || return undef;

    # Put Command follow quit and read Output
    my $socket = $obj->connect() or return error('Could not open the svdrp-socket!');
    print $socket sprintf("%s\r\n", $cmd);
    print $socket sprintf("%s\r\n", 'quit');

    my (@Data, $Line);
	do {
		$Line = <$socket> || 0;
		$Line =~ s/\r\n//g;
		push(@Data, $Line);
	} while($Line);
    $socket->close();

    return \@Data;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $erg = $obj->command('stat disk');
    $console->message($erg);
}

# ------------------
sub scommand {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text = shift || return $console->message(gettext("No Command! Please use scommand 'cmd'"));

    my $erg = $obj->command($text);
    $console->message($erg);
}


1;
