package XXV::MODULES::USER;

use strict;

use Tools;
use Event::File;
use Locale::gettext;


# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'USER',
        Description =>
gettext("This module managed a Useradministration Interface.
for use you can set a Level to the hole Modul with
a parameter 'Level' in the main root or you can set
the same parameter in a function."),
        Version => '0.01',
        Date => '09.09.2004',
        Author => 'xpix',
        Level => 'admin',
        Preferences => {
            Active => {
                description => gettext('Switch the Userauthentification on or off'),
                default     => 'y',
                type        => 'confirm',
            },
        },
        Commands => {
            unew => {
                description => gettext('Create a new User'),
                short       => 'un',
                callback    => sub{ $obj->create(@_) },
            },
            udelete => {
                description => gettext("Delete a user 'uid'"),
                short       => 'ud',
                callback    => sub{ $obj->delete(@_) },
            },
            uedit => {
                description => gettext("Edit a user 'uid'"),
                short       => 'ue',
                callback    => sub{ $obj->edit(@_) },
            },
            ulist => {
                description => gettext("List the users 'uid'"),
                short       => 'ul',
                callback    => sub{ $obj->list(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Look for table or create this table
    my $erg = $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS USER (
          Id int(11) auto_increment NOT NULL,
          Name varchar(100) NOT NULL default '',
          Password varchar(100) NOT NULL,
          Level set('admin', 'user', 'guest' ) NOT NULL,
          PRIMARY KEY  (Id)
        )
    |);

    # The Table is empty? Make a default User ...
    unless($obj->{dbh}->selectall_arrayref('select count(*) from USER')->[0][0]) {
        $obj->insert({
            Name => 'xxv',
            Password => 'xxv',
            Level => 'admin',
        });
    }
}


# ------------------
sub create {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || 0;
    my $data    = shift || 0;

    $obj->edit($watcher, $console, $id, $data);

}

# ------------------
sub edit {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || 0;
    my $data    = shift || 0;

    my $user;
    if($id and not ref $data) {
        $user = $obj->{dbh}->selectrow_hashref("select * from USER where Id = $id")
            or return $console->message(sprintf(gettext("UserID '%s' is not exist in Database!"),$id));

            # question erwartet ein Array
            my @levels = split(/\s*,\s*/, $user->{Levels});
            $user->{Levels} = \@levels;
    }

    my $questions = [
        'Id' => {
            typ     => 'hidden',
            def     => $user->{Id} || 0,
        },
        'Name' => {
            msg   => gettext("Name from this User"),
            req   => gettext('This is required!'),
            def   => $user->{Name} || '',
        },
        'Password' => {
            typ   => 'password',
            msg   => gettext("Password for this User"),
            req   => gettext('This is required!'),
            def   => $user->{Password} || '',
        },
        'Level' => {
            def     => 'guest',
            msg     => gettext("Level for this User"),
            typ     => 'radio',
            req     => gettext('This is required!'),
            choices => ['admin', 'user', 'guest']
        },
    ];

    # Ask Questions
    $data = $console->question(($id ? gettext('Edit User')
				    : gettext('Create User')), $questions, $data);

    if(ref $data eq 'HASH') {
        $obj->insert($data);

        $console->message(gettext('User saved!'));
        $console->link({
            text => gettext("Back to user list"),
            url => "?cmd=ulist",
        }) if($console->typ eq 'HTML');
    }
    return 1;
}

# ------------------
sub delete {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id = shift || return $console->message(gettext("No UserID to delete! Please use udelete 'uid'"));

    my $erg = $obj->{dbh}->do("delete from USER where Id in ($id)")
        or return $console->message(sprintf gettext("UserID '%s' is not exist in Database!"), $id);
    $console->message(sprintf gettext("User %s is deleted."), $id);
    $console->link({
        text => gettext("Back to user list"),
        url => "?cmd=ulist",
    }) if($console->typ eq 'HTML');
}


# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || '';

    my $sql = "select Id, Name, Level from USER";
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    $console->table($erg);
}

# ------------------
sub check {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $name = shift || return;
    my $password = shift || return;

    # check User
    my $sql = sprintf "select * from USER where Name = '%s' and password = md5('%s')", $name, $password;
    $obj->{USER} = $obj->{dbh}->selectrow_hashref($sql);

    if(my $level = $obj->getLevel($obj->{USER}->{Level})) {
        $obj->{USER}->{value} = $level if($level);
    }

    return $obj->{USER};
}

# ------------------
sub allowCommand {
# ------------------
    my $obj = shift || return error ('No Object!' );
    return 1 unless($obj->{Active} eq 'y');

    my $modCfg = shift || return error('No Moduleinformation');
    my $cmdName = shift || return error('No Command name');
    my $user = shift || return error('No User');
    my $DontdumpViolation = shift || '';

    if(
        (exists $modCfg->{Level} and $user->{value} < $obj->getLevel($modCfg->{Level}))
        or
        (exists $modCfg->{Commands}->{$cmdName}->{Level} and $user->{value} < $obj->getLevel($modCfg->{Commands}->{$cmdName}->{Level}))
    ) {
        error(sprintf('User %s with Level %s has try to call command %s without permissions!',
            $user->{Name}, $user->{Level}, $cmdName))
		if($DontdumpViolation eq '');
        return 0;
    }
    return 1;
}


# ------------------
sub getLevel {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $name = shift || return;

    # Level Table
    $obj->{LEV} = {
        admin   => 10,
        user    => 5,
        guest   => 1,
    } unless(exists $obj->{LEV});

    if($obj->{LEV}->{$name}) {
        return $obj->{LEV}->{$name};
    } else {
        return error("This Levelname '$name' is not exist");
    }

}

# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;

    if(ref $data eq 'HASH') {
        my ($names, $vals, $kenn);
        map {
            push(@$names, $_);
            push(@$vals, $data->{$_});
            if($_ eq 'Password') {
                push(@$kenn, 'MD5(?)');
            } else {
                push(@$kenn, '?');
            }
        } sort keys %$data;

        my $sql = sprintf("REPLACE INTO USER (%s) VALUES (%s)",
                join(', ', @$names),
                join(', ', @$kenn),
        );
        my $sth = $obj->{dbh}->prepare( $sql );
        $sth->execute( @$vals );
    } else {
        my $sth = $obj->{dbh}->prepare('REPLACE INTO USER VALUES (?,?,?,?)');
        $sth->execute( @$data );
    }
}

1;
